/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cdt.internal.ui.cview;

import java.util.Iterator;
import java.util.List;

import org.eclipse.cdt.core.model.ICElement;
import org.eclipse.cdt.core.model.ICProject;
import org.eclipse.cdt.internal.core.model.CModelManager;
import org.eclipse.cdt.internal.core.model.ProcessorContainer;
import org.eclipse.cdt.internal.ui.actions.SelectionConverter;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.TextActionHandler;

/**
 * This is the action group for refactor actions,
 * including global action handlers for copy, paste and delete.
 * 
 */
public class TreeCViewRefactorActionGroup extends TreeCViewActionGroup {

	private Clipboard clipboard;

	private CopyAction copyAction;
	private TreeCViewDeleteAction deleteAction;
	private PasteAction pasteAction;
	private TreeCViewRenameAction renameAction;
	private CViewMoveAction moveAction;
	private TextActionHandler textActionHandler;
	
	public TreeCViewRefactorActionGroup(TreeCView cview) {
		super(cview);
	}

	public void dispose() {
		if (clipboard != null) {
			clipboard.dispose();
			clipboard = null;
		}
		super.dispose();
	}

	public void fillContextMenu(IMenuManager menu) {
		IStructuredSelection celements = (IStructuredSelection) getContext().getSelection();
		IStructuredSelection selection = SelectionConverter.convertSelectionToResources(celements);

		boolean anyResourceSelected = !selection.isEmpty()
				&& SelectionConverter.allResourcesAreOfType(selection, IResource.PROJECT | IResource.FOLDER | IResource.FILE);

		copyAction.selectionChanged(selection);
		menu.add(copyAction);
		pasteAction.selectionChanged(selection);
		menu.add(pasteAction);

		if (anyResourceSelected) {
//			deleteAction.selectionChanged(selection);
//			menu.add(deleteAction);
//			moveAction.selectionChanged(selection);
//			menu.add(moveAction);
			renameAction.selectionChanged(selection);
			menu.add(renameAction);
		}
		// add Delete action if selection includes any software platform projects with dependent projects
		if (canAddDeleteAction(selection)) {
			deleteAction.selectionChanged(selection);
			menu.add(deleteAction);
		}
	}

	public void fillActionBars(IActionBars actionBars) {
		textActionHandler = new TextActionHandler(actionBars); // hooks handlers
		textActionHandler.setCopyAction(copyAction);
		textActionHandler.setPasteAction(pasteAction);
		textActionHandler.setDeleteAction(deleteAction);
		renameAction.setTextActionHandler(textActionHandler);
		
		actionBars.setGlobalActionHandler(ActionFactory.MOVE.getId(), moveAction);
		actionBars.setGlobalActionHandler(ActionFactory.RENAME.getId(), renameAction);
	}

	/**
 	 * Handles a key pressed event by invoking the appropriate action.
 	 */
	public void handleKeyPressed(KeyEvent event) {
		if (event.character == SWT.DEL && event.stateMask == 0) {
			if (deleteAction.isEnabled()) {
				deleteAction.run();
			}
			// Swallow the event.
			event.doit = false;
		} else if (event.keyCode == SWT.F2 && event.stateMask == 0) {
			if (renameAction.isEnabled()) {
				renameAction.run();
			}
			// Swallow the event.
			event.doit = false;			
		}
	}

	protected void makeActions() {
		TreeViewer treeViewer = getCView().getViewer();
		Shell shell = getCView().getSite().getShell();
		clipboard = new Clipboard(shell.getDisplay());
		
		pasteAction = new PasteAction(shell, clipboard);
		ISharedImages images = PlatformUI.getWorkbench().getSharedImages();
		pasteAction.setDisabledImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE_DISABLED));
		pasteAction.setImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
		pasteAction.setHoverImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));

		copyAction = new CopyAction(shell, clipboard, pasteAction);
		copyAction.setDisabledImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_COPY_DISABLED));
		copyAction.setImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		copyAction.setHoverImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		
		moveAction = new CViewMoveAction(shell, treeViewer);
		renameAction = new TreeCViewRenameAction(shell, treeViewer);
		
		deleteAction = new TreeCViewDeleteAction(shell);
		deleteAction.setDisabledImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE_DISABLED));
		deleteAction.setImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));		
		deleteAction.setHoverImageDescriptor(images.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
	}

	public void updateActionBars() {
		IStructuredSelection celements = (IStructuredSelection) getContext().getSelection();
		IStructuredSelection selection = SelectionConverter.convertSelectionToResources(celements);

		copyAction.selectionChanged(selection);
		pasteAction.selectionChanged(selection);
		deleteAction.selectionChanged(selection);
		moveAction.selectionChanged(selection);
		renameAction.selectionChanged(selection);
	}
	
	private boolean canAddDeleteAction(IStructuredSelection selection) {
		IResource resource;
		for (Iterator it = selection.iterator(); it.hasNext();) {
			resource = (IResource)it.next();
			if (resource.getType() == IResource.PROJECT) {
				IProject proj = resource.getProject();
				try {
					IProjectDescription desc;
					ProcessorContainer pContainer;
					
					if (proj.hasNature("com.xilinx.mdt.edk.swPlatformProject")) {
						desc = proj.getDescription();
						pContainer = CModelManager.getDefault().getProcessorContainer(desc.getComment());
						if (pContainer != null) {
							List list = pContainer.getChildrenOfType(ICElement.C_PROJECT);
							ICProject cproj;
							IProject project;
							for (Iterator cit = list.iterator(); cit.hasNext();) {
								cproj = (ICProject)cit.next();
								project = cproj.getProject();
								if (cproj.getElementName().equals(proj.getName())
										|| project.hasNature("com.xilinx.mdt.edk.swPlatformProject")
										|| project.hasNature("com.xilinx.mdt.edk.bspProject")) {
									continue;
								}
								IProject[] refProjs = project.getDescription().getReferencedProjects();
								for (int i = 0; i < refProjs.length; i++) {
									if (refProjs[i].getName().equals(proj.getName())) {
										return false;
									}
								}
							}
						}
					}
				} catch(CoreException ex) {
					//System.out.println("canDeleteAction: " + ex.getMessage());
				}
			}
		}
		return true;
	}
}
